
#ifndef KIDENTITYMANAGEMENT_EXPORT_H
#define KIDENTITYMANAGEMENT_EXPORT_H

#ifdef KIDENTITYMANAGEMENT_STATIC_DEFINE
#  define KIDENTITYMANAGEMENT_EXPORT
#  define KIDENTITYMANAGEMENT_NO_EXPORT
#else
#  ifndef KIDENTITYMANAGEMENT_EXPORT
#    ifdef KF5IdentityManagement_EXPORTS
        /* We are building this library */
#      define KIDENTITYMANAGEMENT_EXPORT __declspec(dllexport)
#    else
        /* We are using this library */
#      define KIDENTITYMANAGEMENT_EXPORT __declspec(dllimport)
#    endif
#  endif

#  ifndef KIDENTITYMANAGEMENT_NO_EXPORT
#    define KIDENTITYMANAGEMENT_NO_EXPORT 
#  endif
#endif

#ifndef KIDENTITYMANAGEMENT_DEPRECATED
#  define KIDENTITYMANAGEMENT_DEPRECATED __attribute__ ((__deprecated__))
#endif

#ifndef KIDENTITYMANAGEMENT_DEPRECATED_EXPORT
#  define KIDENTITYMANAGEMENT_DEPRECATED_EXPORT KIDENTITYMANAGEMENT_EXPORT KIDENTITYMANAGEMENT_DEPRECATED
#endif

#ifndef KIDENTITYMANAGEMENT_DEPRECATED_NO_EXPORT
#  define KIDENTITYMANAGEMENT_DEPRECATED_NO_EXPORT KIDENTITYMANAGEMENT_NO_EXPORT KIDENTITYMANAGEMENT_DEPRECATED
#endif

#if 0 /* DEFINE_NO_DEPRECATED */
#  ifndef KIDENTITYMANAGEMENT_NO_DEPRECATED
#    define KIDENTITYMANAGEMENT_NO_DEPRECATED
#  endif
#endif

#endif
